/* setup.c - WinSNTPC Installer
 * Copyright(C) 2002  Yuichi Kobayashi <kobayasy@kobayasy.com>
 *
 * This program comes with NO WARRANTY, to the extent permitted by law.
 * You may redistribute it under the terms of the GNU General Public License;
 * see the file named LICENSE.txt for details.
 */

#include <windows.h>
#include <shlobj.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "setup.h"

#define P         "-"
#define PTIME     P"time"
#define PPERIODIC P"periodic="
#define PQUIET    P"quiet"
#define PVERBOCE  P"verboce"

#define NSTR 1024

static struct {
  char name[NSTR], fname[NSTR];
  char host[NSTR];
  enum {Sntpc, Timec} protocol;
  unsigned long periodic, interval;
  enum {Startup, Desktop} folder;
} g = {
  "WinSNTPC", "winsntpc.exe",
  "",
  Sntpc,
  0, 168,
  Startup
};

static int execute_command(char *commandline, unsigned long *exitcode) {
  int status = -1;
  STARTUPINFO si;
  PROCESS_INFORMATION pi;

  memset(&si, 0, sizeof(si));
  si.cb = sizeof(si);
  if(CreateProcess(NULL, commandline,
                   NULL, NULL, TRUE, 0, NULL, NULL, &si, &pi ) == FALSE)
    status = 1;
  else if(!exitcode)
    status = 0;
  else if(WaitForSingleObject(pi.hProcess, INFINITE) == WAIT_FAILED) {
    TerminateProcess(pi.hProcess, -1);
    status = 2;
  }
  else if(GetExitCodeProcess(pi.hProcess, exitcode) == FALSE)
    status = 3;
  else
    status = 0;
  return status;
}

static int link_folder(char *name, char *fname, char *args,
                       int folder ) {
  int status = -1;
  ITEMIDLIST *pidl;
  char linkpath[NSTR];
  IShellLink* psl;
  IPersistFile* ppf;
  char linkfile[NSTR];
  unsigned short linkfile_w[(NSTR+1)/2];

  if(SHGetSpecialFolderLocation(NULL, folder, &pidl) != NOERROR)
    status = 1;
  else if(SHGetPathFromIDList(pidl, linkpath) == FALSE)
    status = 2;
  else if(CoInitialize(NULL) != S_OK)
    status = 3;
  else {
    if(CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
                        &IID_IShellLink, (void **)&psl ) != S_OK)
      status = 4;
    else {
      if(psl->lpVtbl->SetPath(psl, fname) != NOERROR)
        status = 5;
      else if(psl->lpVtbl->SetArguments(psl, args) != NOERROR)
        status = 6;
      else if(psl->lpVtbl->QueryInterface(psl, &IID_IPersistFile,
                                          (void **)&ppf ) != S_OK)
        status = 7;
      else {
        if(sprintf(linkfile, "%s\\%s.lnk", linkpath, name),
           !MultiByteToWideChar(CP_ACP, 0, linkfile, -1,
                                linkfile_w, (NSTR+1)/2-1 ) )
          status = 8;
        else if(ppf->lpVtbl->Save(ppf, linkfile_w, TRUE) != S_OK)
          status = 9;
        else
          status = 0;
        ppf->lpVtbl->Release(ppf);
      }
      psl->lpVtbl->Release(psl);
    }
    CoUninitialize();
  }
  return status;
}

static int init_args(HWND hwndDlg) {
  int status = 0;
  char buffer[NSTR], *s;

  GetModuleFileName(NULL, buffer, NSTR-1);
  if(s = strrchr(buffer, '\\'))
    strcpy(++s, g.fname), strcpy(g.fname, buffer);
  SetDlgItemText(hwndDlg, ID_HOST, g.host);
  CheckDlgButton(hwndDlg, ID_NTP,
                 (g.protocol == Sntpc) ? BST_CHECKED : BST_UNCHECKED );
  CheckDlgButton(hwndDlg, ID_TIME,
                 (g.protocol == Timec) ? BST_CHECKED : BST_UNCHECKED );
  CheckDlgButton(hwndDlg, ID_STARTUP, ((g.folder == Startup) &&
                                       !g.periodic ) ?
                                         BST_CHECKED : BST_UNCHECKED );
  CheckDlgButton(hwndDlg, ID_DESKTOP, ((g.folder == Desktop) &&
                                       !g.periodic ) ?
                                         BST_CHECKED : BST_UNCHECKED );
  CheckDlgButton(hwndDlg, ID_PERIODIC,
                 g.periodic ? BST_CHECKED : BST_UNCHECKED );
  EnableWindow(GetDlgItem(hwndDlg, ID_INTERVAL),
               g.periodic ? TRUE : FALSE );
  sprintf(buffer, "%lu", g.interval);
  SetDlgItemText(hwndDlg, ID_INTERVAL, buffer);
  return status;
}
static int make_args(HWND hwndDlg, HINSTANCE hInstance, char *args) {
  int status = 0;
  char buffer[NSTR];
  signed long n;
  char *e;

  *args = 0;
  GetDlgItemText(hwndDlg, ID_HOST, g.host, NSTR-1);
  if(!*g.host) {
    LoadString(hInstance, ID_MESSAGE1, buffer, NSTR);
    strcat(args, buffer);
    ++status;
  }
  else if(strpbrk(g.host, " \t")) {
    LoadString(hInstance, ID_MESSAGE2, buffer, NSTR);
    strcat(args, buffer);
    ++status;
  }
  else if(!strncmp(g.host, P, strlen(P))) {
    LoadString(hInstance, ID_MESSAGE3, buffer, NSTR);
    strcat(args, buffer);
    ++status;
  }
  g.protocol = (IsDlgButtonChecked(hwndDlg, ID_TIME)
                == BST_CHECKED ) ? Timec : Sntpc;
  if(IsDlgButtonChecked(hwndDlg, ID_PERIODIC) != BST_CHECKED)
    g.periodic = 0;
  else if(GetDlgItemText(hwndDlg, ID_INTERVAL, buffer, NSTR-1),
          n = strtol(buffer, &e, 10),
          (n > 0) && !*e )
    g.periodic = n;
  else {
    LoadString(hInstance, ID_MESSAGE4, buffer, NSTR);
    strcat(args, buffer);
    ++status;
  }
  g.folder = (IsDlgButtonChecked(hwndDlg, ID_DESKTOP)
              == BST_CHECKED ) ? Desktop : Startup;
  if(!status) {
    if(g.periodic)
      sprintf(buffer, " %s%lu", PPERIODIC, g.periodic);
    else
      *buffer = 0;
    sprintf(args, "%s%s%s%s%s",
                  g.host,
                  (g.protocol == Timec) ? " "PTIME : "",
                  buffer,
                  (g.folder == Desktop) ? " "PVERBOCE : "",
                  g.periodic ? " "PQUIET : "" );
  }
  return status;
}

static BOOL CALLBACK AboutDialog(HWND hwndDlg, UINT uMsg,
                                 WPARAM wParam, LPARAM lParam ) {
  BOOL result = FALSE;
  char buffer[NSTR];

  switch(uMsg) {
   case WM_INITDIALOG:
    GetDlgItemText(hwndDlg, ID_BUILD, buffer, NSTR-1);
    strcat(buffer, __DATE__);
    SetDlgItemText(hwndDlg, ID_BUILD, buffer);
    result = TRUE;
    break;
   case WM_CLOSE:
    EndDialog(hwndDlg, 0);
    result = TRUE;
    break;
  }
  return result;
}
static BOOL CALLBACK MainDialog(HWND hwndDlg, UINT uMsg,
                                WPARAM wParam, LPARAM lParam ) {
  BOOL result = FALSE;
  static HINSTANCE hInstance;
  static HMENU hmenu;
  char args[NSTR];
  char buffer[NSTR];
  unsigned long status;

  switch(uMsg) {
   case WM_INITDIALOG:
    hInstance = (HINSTANCE)GetWindowLong(hwndDlg, GWL_HINSTANCE);
    hmenu = CreateMenu();
    AppendMenu(hmenu, MF_STRING, ID_ABOUT, "&About");
    SetMenu(hwndDlg, hmenu);
    init_args(hwndDlg);
    result = TRUE;
    break;
   case WM_DESTROY:
    DestroyMenu(hmenu);
    result = TRUE;
    break;
   case WM_CLOSE:
    EndDialog(hwndDlg, 0);
    result = TRUE;
    break;
   case WM_COMMAND:
    switch(LOWORD(wParam)) {
     case ID_ABOUT:
      DialogBox(hInstance, "ABOUTDIALOG", hwndDlg, AboutDialog);
      result = TRUE;
      break;
     case ID_CANCEL:
      EndDialog(hwndDlg, 0);
      result = TRUE;
      break;
     case ID_NTP:
      if(IsDlgButtonChecked(hwndDlg, ID_NTP) != BST_CHECKED) {
        CheckDlgButton(hwndDlg, ID_NTP,  BST_CHECKED);
        CheckDlgButton(hwndDlg, ID_TIME, BST_UNCHECKED);
      }
      result = TRUE;
      break;
     case ID_TIME:
      if(IsDlgButtonChecked(hwndDlg, ID_TIME) != BST_CHECKED) {
        CheckDlgButton(hwndDlg, ID_NTP,  BST_UNCHECKED);
        CheckDlgButton(hwndDlg, ID_TIME, BST_CHECKED);
      }
      result = TRUE;
      break;
     case ID_STARTUP:
      if(IsDlgButtonChecked(hwndDlg, ID_STARTUP) != BST_CHECKED) {
        CheckDlgButton(hwndDlg, ID_STARTUP,  BST_CHECKED);
        CheckDlgButton(hwndDlg, ID_DESKTOP,  BST_UNCHECKED);
        CheckDlgButton(hwndDlg, ID_PERIODIC, BST_UNCHECKED);
        EnableWindow(GetDlgItem(hwndDlg, ID_INTERVAL), FALSE);
      }
      result = TRUE;
      break;
     case ID_DESKTOP:
      if(IsDlgButtonChecked(hwndDlg, ID_DESKTOP) != BST_CHECKED) {
        CheckDlgButton(hwndDlg, ID_STARTUP,  BST_UNCHECKED);
        CheckDlgButton(hwndDlg, ID_DESKTOP,  BST_CHECKED);
        CheckDlgButton(hwndDlg, ID_PERIODIC, BST_UNCHECKED);
        EnableWindow(GetDlgItem(hwndDlg, ID_INTERVAL), FALSE);
      }
      result = TRUE;
      break;
     case ID_PERIODIC:
      if(IsDlgButtonChecked(hwndDlg, ID_PERIODIC) != BST_CHECKED) {
        CheckDlgButton(hwndDlg, ID_STARTUP,  BST_UNCHECKED);
        CheckDlgButton(hwndDlg, ID_DESKTOP,  BST_UNCHECKED);
        CheckDlgButton(hwndDlg, ID_PERIODIC, BST_CHECKED);
        EnableWindow(GetDlgItem(hwndDlg, ID_INTERVAL), TRUE);
      }
      result = TRUE;
      break;
     case ID_INSTALL:
      if(make_args(hwndDlg, hInstance, args))
        MessageBox(hwndDlg, args, TITLE, MB_OK|MB_ICONEXCLAMATION);
      else if(link_folder(g.name, g.fname, args,
                          (g.folder == Desktop) ? CSIDL_DESKTOP
                                                : CSIDL_STARTUP )) {
        LoadString(hInstance, ID_MESSAGE5, buffer, NSTR);
        MessageBox(hwndDlg, buffer, TITLE, MB_OK|MB_ICONEXCLAMATION);
      }
      else {
        LoadString(hInstance, (g.folder == Desktop) ? ID_MESSAGE7
                                                    : ID_MESSAGE6,
                              buffer, NSTR );
        MessageBox(hwndDlg, buffer, TITLE, MB_OK);
        EndDialog(hwndDlg, 0);
      }
      result = TRUE;
      break;
     case ID_TEST:
      if(make_args(hwndDlg, hInstance, args))
        MessageBox(hwndDlg, args, TITLE, MB_OK|MB_ICONEXCLAMATION);
      else if(sprintf(buffer, "%s %s %s%lu %s",
                              g.fname, args, PPERIODIC, 0, PQUIET ),
              execute_command(buffer, &status) || status ) {
        LoadString(hInstance, ID_MESSAGE8, buffer, NSTR);
        MessageBox(hwndDlg, buffer, TITLE, MB_OK|MB_ICONEXCLAMATION);
      }
      else {
        LoadString(hInstance, ID_MESSAGE9, buffer, NSTR);
        MessageBox(hwndDlg, buffer, TITLE, MB_OK);
      }
      result = TRUE;
      break;
    }
    break;
  }
  return result;
}
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpszCmdLine, int nCmdShow ) {
  return DialogBox(hInstance, "MAINDIALOG", NULL, MainDialog);
}
